__author__ = 'kim'

from pdtrend.filling_missing_data import FMdata
from pdtrend.detrend import PDTrend
from pdtrend.test.run import test

from pdtrend.utils.logger import Logger
from pdtrend.datasets.base import load_lightcurve_set




#!/usr/bin/env python

'''
Libraries for PDtrending algorithm. The major part of the libraries
consist of clustering routine.
'''

import sys
import math
from numpy import *

import rpy
from rpy import r

from misclib import *

sys.path.append('./')

def Pycluster_to_hcluster(tree):
	'''
	Transform tree structure of Pycluster to hcluster to use 
	dendrogram function of hcluster which plots dendrogram.
	
	tree :
		Tree from Pycluster.
	
	return :
		hcluster tree.
	'''
	
	hcluster_tree = []
	len_tree = len(tree) + 1
	i = 0
	for node in tree:
		hcluster_node = []
		#make new node for hcluster
		node_left = node.left
		node_right = node.right
		#modify index of Pycluster for hcluster.
		if node_left < 0:
			node_left = (node_left + 1) * -1 + len_tree
		if node_right < 0:
			node_right = (node_right + 1) * -1 + len_tree
		
		#sort
		if node_left > node_right:
			dummy = node_right
			node_right = node_left
			node_left = dummy
		
		#count number of whole elements below each node.
		s_group = []
		simplify_group(find_group_with_node_index(tree, i), s_group)
			
		hcluster_node.append([node_left, node_right, node.distance, len(s_group)])
		hcluster_tree.append(hcluster_node[0])
		i += 1
	hcluster_tree = array(hcluster_tree)
	#print hcluster_tree
	
	return hcluster_tree

def remove_small(groups, size=9):
	'''
	Remove groups smaller than size.
	
	groups :
		List of group.
	
	size :
		Minimum size of group.
	'''
	
	new_groups = []
	
	for group in groups:
		if len(group) > size:
			new_groups.append(group)
			
	return new_groups

def find_group_with_distance(tree, distance):
	'''
	Find group from tree based on the index.
	
	tree :
		Tree structure returned from Pycluster module
		
	distance :
		Maximum distance within group.
		
	return :
		Multiple groups within the distance.
	'''
	
	groups = []
	
	#get node within the distance
	node_list = []
	for i in range(len(tree)):
		if tree[i].distance > distance:
			break
		node_list.append(i)
	
	#make group.
	temp_groups = []
	for node_index in node_list:
		group = find_group_with_node_index(tree, node_index)
		left_group = []
		right_group = []
		simplify_group(group[0], left_group)
		simplify_group(group[1], right_group)
		group = left_group + right_group
		
		temp_groups.append(group)
		
	#check overlapped group and remove.
	groups = remove_subset_cluster(temp_groups)
	
	return groups

def find_group_with_node_index(tree, index):
	'''
	Find group from tree based on the index of node.
	
	tree :
		Tree structure returned from Pycluster module
		
	index :
		Real index of tree.
		
	return :
		Two groups divided by index.
	'''
	
	group = []
	node = tree[index]
	eles = [node.left, node.right]
	for ele in eles:
		if ele >= 0:
			group.append([ele])
		else:
			group.append(find_group_with_link_index(tree, ele))
			
	return group

def find_group_with_link_index(tree, index):
	'''
	Find group from tree based on the index of link.
	Value of link is smaller than -1. -1 means the element
	is linked with 0th node. -2 means it is linked with 1th node and so on.
	
	tree :
		Tree structure returned from Pycluster module
		
	index :
		Linkage index less than -1.
		
	return :
		Two groups divided by index.
	'''
	
	group = []
	node = tree[index * -1 - 1]
	eles = [node.left, node.right]
	for ele in eles:
		if ele >= 0:
			group.append([ele])
		else:
			group.append(find_group_with_link_index(tree, ele))
			
	return group
	
	
def simplify_group(group, s_group):
	'''
	Unify multiple lists inside group into one list.
	
	group :
		Looks like [[6, 4], 3]
	
	s_group :
		simplified group looks like [6, 4, 3]
		
	return :
		None
	'''
	
	if isinstance(group, int):
		s_group.append(group)
		return
	else:
		for ele in group:
			simplify_group(ele, s_group)

def find_group_DW(tree, dist_matrix, mes=0, seed_max=0, l_significance=0.1):
	'''
	For more details, see the Kim et al. 2008.
		
	tree :
		Tree structure returned from Pycluster module.
	
	dist_matrix :
		Distance matrix (= 1. - correlation matrix)
	
	mes = 0 :
		Total number of measurement of each light curve.
	
	seed_max = 0 :
		To get more tighter seed. 1 ~ 10 are good values. '10' gets more tighter clusters than '1'.
		
	return :
		List of clusters.
	'''
	
	r.library('nortest')
	
	clusters = []
	#print tree, len(tree)
	density_list = []
	for i in range(len(dist_matrix) - 1):
		for j in range(i + 1, len(dist_matrix)):
			density_list.append(dist_matrix[i][j])
	density_list_clip = sigma_clipping(density_list, sigma=3.)
	overall_density = (max(density_list_clip) - min(density_list_clip)) / len(dist_matrix)
	#print overall_density, mean(density_list_clip), std(density_list_clip)
	
	#get highly correlated pair of elements.
	initial_seed = []
	for i in range(len(tree)):
		#both left and right element has to be star. not a link to other cluster.
		if tree[i].left >= 0 and tree[i].right >= 0:
			#to get more tight elements.
			if dist_matrix[tree[i].left][tree[i].right] <= median(density_list_clip) / seed_max:
				if mes == 0:
					initial_seed.append(i)
				elif dist_matrix[tree[i].left][tree[i].right] <= (1. - 3. / math.sqrt(mes)):
					initial_seed.append(i)
	#print initial_seed
	
	#start from highly correlated initial pair.
	for i in initial_seed:
		#print tree[i]
		current_node = i
		while current_node < len(tree) - 1:
			cluster_1 = []
			cluster_2 = []
			#find base cluster --> cluster_1
			simplify_group(find_group_with_node_index(tree, current_node), cluster_1)
			#find cluster which will be merged --> cluster_2
			dummy = find_one_side_group(tree, (current_node + 1) * -1)
			current_node = dummy[0]
			simplify_group(dummy[1], cluster_2)
			
			#check the density changes with overall density
			#initial density
			d_1 = []
			for ele_i in range(len(cluster_1) - 1):
				for ele_j in range(ele_i + 1, len(cluster_1)):
					if ele_i != ele_j:
						d_1.append(dist_matrix[cluster_1[ele_i]][cluster_1[ele_j]])
			#density after merged
			d_merge = []
			cluster_3 = hstack([cluster_1, cluster_2])
			for ele_i in range(len(cluster_3) - 1):
				for ele_j in range(ele_i + 1, len(cluster_3)):
					if ele_i != ele_j:
						d_merge.append(dist_matrix[cluster_3[ele_i]][cluster_3[ele_j]])
			
			d_1 = array(d_1)
			d_merge = array(d_merge)
			if len(d_merge) < 8:
				continue
			else:
				#the resulting clusters are almost identical. not use anymore.
				#d_merge = array(d_merge)
				#d_merge = .5 * log((1. + d_merge) / (1. - d_merge))
				
				ad = r.ad_test(d_merge)
				ad_p = ad['p.value']
				p_value = ad_p
				
				#check the level of significance
				#if it's out of normality, the previous cluster is the final cluster.
				if p_value < l_significance:
					#becausd AD test needs at least 8 elements.
					if len(cluster_1) >= 5:
						#print cluster_1
						clusters.append(cluster_1)
					break
				#it's still gaussian, but if there comes outliers into clusters, stop it.
				#the resulting clusters are almost identical. not use anymore.
				#elif len(d_1[where(d_1 > mean(density_list_clip))]) > 0:
				#	if len(cluster_1) >= 5:
				#		clusters.append(cluster_1)
				#	break
	
	return clusters

def find_one_side_group(tree, index):
	'''
		Return group of one side.
		
		tree :
			Tree structure returned from Pycluster module
			
		index :
			Index of tree structure. This routine first find node of index 
			which looks like [index, other_index or node_value]
			and, return the group of [other_index] or [value]
	'''
	
	for i in range(len(tree)):
		if tree[i].left == index:
			group_index = tree[i].right
			break
		if  tree[i].right == index:
			group_index = tree[i].left
			break
	
	if group_index >=0 :
		return [i, [group_index]]
	else:
		return [i, find_group_with_link_index(tree, group_index)]

def cal_correlation_matrix(whole_lc, bin_size=0):
	'''
	Calculate the Pearson correlation matrix.
	
	whole_lc : 
		List of light curves.
	
	bin_size :
		Binning window size. to reduce noise.
		Binning the data only when calculated correlation values.
		Will not change original light curves.
	
	return :
		The Pearson correlation magtrix.
	'''
	
	corr_list = ones((len(whole_lc), len(whole_lc)))
	for i in range(len(whole_lc) - 1):
		#print '	#Correlation values of %dth star is calculating..' % (i + 1)
		for j in range(i, len(whole_lc)):
			if bin_size == 0:
				pear_corr = corrcoef(whole_lc[i], whole_lc[j])[0][1]
			else:
				pear_corr = corrcoef(bin_lc(whole_lc[i], bin_size)[0], bin_lc(whole_lc[j], bin_size)[0])[0][1]
			corr_list[i, j] = pear_corr
			corr_list[j, i] = pear_corr
	
	return corr_list
	
def get_detrened_lc_set(lc, trend_set):
	'''
	Return de-trended lc by using trends set.
	We use Multiple Linear Regression Method to de-trend.
	
	lc :
		Original light curve of flux.
	
	trend_set :
		Set of trend light curves constructed by create_trend routine.
	
	return :
		De-trended light curve.
	'''
	
	#Multiple linear regression method, least square method
	X = ones([len(trend_set[0]), len(trend_set) + 1])
	X[:, 0] = [1]
	X[:, 1:] = transpose(trend_set)
	beta_1 = linalg.inv(dot(transpose(X), X))
	beta_2 = dot(transpose(X), lc)
	beta = dot(beta_1, beta_2)
	
	return lc - dot(X, beta)

def get_quadprog(lc, trend_set):
	'''
	Return de-trended lc by quadratic programming.
	It constraints the free parameters to be bigger than 0.
	See Kim et al. 2008 for more details.
	
	lc :
		Original light curve of flux.
	
	trend_set :
		Set of trend light curves constructed by create_trend routine.
	
	return :
		De-trended light curve.
	'''
	
	r.library('quadprog')
	
	X = transpose(trend_set)
	dmat = r.crossprod(X, X)
	dvec = r.crossprod(lc, X)
	
	results = r.solve_QP(dmat, dvec, r.diag(len(trend_set)))
	#print results['solution'], results['value']
	
	return lc - dot(results['solution'], trend_set)

def get_linprog(lc, trend_set):
	'''
	Return de-trended lc by linear programming.
	It constraints the free parameters to be bigger than 0.
	
	lc :
		Original light curve of flux.
	
	trend_set :
		Set of trend light curves constructed by create_trend routine.
	
	return :
		De-trended light curve.
	'''
	
	r.library('linprog')
	
	X = transpose(trend_set)
	#dmat = r.crossprod(X, X)
	dvec = r.crossprod(lc, X)
	
	results = r.solveLP(dvec, zeros([len(trend_set)]), r.diag(len(trend_set)))
	print results['opt'], results['solution']
	sys.exit()
	
	#return lc - dot(results['solution'], trend_set)
	
def kovacs(lc, trend_set):
	'''
	Apply TFA with determined trends by our algorithm.
	See Kovacs et al. 2005 for more detail.
	
	lc :
		Original light curve of flux
	
	trend_set :
		Set of trend light curves constructed by create_trend routine.
	
	return :
		De-trended light curve.
	'''
	
	dup_lc = lc[::]
	dup_trend = trend_set[::]
	
	dup_lc -= mean(dup_lc)
	for i in range(len(dup_trend)):
		dup_trend[i] -= mean(dup_trend[i])
	
	g = zeros((len(dup_trend), len(dup_trend)))
	h = zeros((len(dup_trend)))
	c = zeros((len(dup_trend)))
	for i in range(len(dup_trend)):
		for j in range(len(dup_trend)):
			g[i, j] = sum(dot(dup_trend[i], dup_trend[j]))
	G = linalg.inv(g)
	
	for i in range(len(dup_trend)):
		h[i] = sum(dot(dup_lc, dup_trend[i]))
	for i in range(len(dup_trend)):
		c[i] = sum(dot(G[i:i + 1], h))

	trend = dot(c, dup_trend)
	detrended = dup_lc - trend
	
	return detrended
	
def create_trend(template_set, whole_lc):
	'''
	Create trend with selected template light curves.
	We use weighted sum of normlized light curves by median values of the lc.
	See Kim et al. 2008 for more details.
	
	template_set :
		Indices of template light curves.
	
	whole_lc : 
		Light curves of every stars.
		
	return :
		Constructed trend.
	'''
	
	trend = []
	trend_lc = []
	
	#normalized by mean value
	normalized = []
	for i in template_set:
		normalized.append((whole_lc[i] - mean(whole_lc[i])) / std(whole_lc[i]))
	
	#weighting by sigma^2
	std_inv_list = []
	for i in range(len(normalized)):
		std_inv_list.append(1. / std(normalized[i])**2.)
	weight_list = std_inv_list / sum(std_inv_list)
	
	for i in range(len(normalized)):
		trend_lc.append(normalized[i] * weight_list[i])
	trend_lc = array(trend_lc)
	
	#make trend..
	for i in range(len(trend_lc[0])):
		#trend.append(median(trend_lc[:, i]))
		trend.append(sum(trend_lc[:, i]))
	
	return array(trend)

def remove_subset_cluster(cluster_list):
	'''
	Remove subset clusters which are included in other clusters.
	
	cluster_list : 
		Initial list of clusters.
		
	return :
		List of clusters after removal of subsets.
	'''
	
	cluster_list.sort(cmp=cmp_length)
	dup = cluster_list[::]
	
	new_cluster_list = []
	#print dup
	
	#check sub-set.
	for i in range(len(dup)):
		if_duplicated = 0
		cluster_d = dup[i]
		len_cluster_d = len(cluster_d)
		#print 'd:', cluster_d
		for j in range(len(cluster_list)):
			if i == j:
				break
			cluster_o = dup[j]
			len_cluster_o = len(cluster_o)
			#print 'o:', cluster_o
			if len_cluster_d <= len_cluster_o:
				duplicated = [ele for ele in cluster_d if ele in cluster_o]
				if len(duplicated) == len_cluster_d:
					if_duplicated = 1
					break
		
		if if_duplicated == 0:
			new_cluster_list.append(cluster_d)
	
	return new_cluster_list

def remove_duplicated_cluster(corr, cluster_list):
	'''
	Make clusters simple by removing duplicated members through all clusters.
	
	corr :
		The Pearson correlation matrix.
	
	cluster_list :
		Initial cluster list.
	
	return :
		list of cluster after removal of dup. clusters.
	'''
	
	dup=corr.copy()
	
	#make a member of cluster list.
	member_list=[]
	for cluster in cluster_list:
		for member in cluster:
			if not member in member_list:
				member_list.append(member)
	
	#check if each member are overlapped through multiple cluster.
	for member in member_list:
		mem_corr_list={}
		cluster_index=0
		for cluster in cluster_list:
			if len(cluster)==2:
				cluster_index+=1
				continue
			#calculate average correlation if overlapped.
			sum_corr=0.
			avg_corr=0.
			if member in cluster:
				for cluster_member in cluster:
					if member!=cluster_member:
						sum_corr+=dup[member, cluster_member]
				avg_corr=sum_corr/(len(cluster)-1)
				mem_corr_list[cluster_index]=avg_corr
			cluster_index+=1
		#print member, mem_corr_list, max(mem_corr_list)
		
		#remain only one highest correlated value, remove else others.
		if len(mem_corr_list)>1:
			max_key=max(mem_corr_list)
			for key, val in mem_corr_list.iteritems():
				#print key, cluster_list[key]
				if key!=max_key:
					cluster_list[key].pop([j for j in range(len(cluster_list[key])) if cluster_list[key][j]==member][0])
	
	return cluster_list
	
if __name__=='__main__':
	print 'Function set for PDtrending algorithm'



#!/usr/bin/env python

help = '''
Photometric De-Trending (pdtrend, also Pavlos-Daewon de-Trending) pipeline for astronomical time series. See details in Kim et al. 2008.


Packages to run this pipeline :
	matplotlib : http://matplotlib.sourceforge.net/
	numpy : http://numpy.scipy.org/
	Pycluster : http://bonsai.ims.u-tokyo.ac.jp/~mdehoon/software/cluster/
	hcluster : http://code.google.com/p/scipy-cluster/
	R : http://www.r-project.org/
		You need to install 'nortest' abd 'quadprog' package at R. Use 'install.packages' command in R shell.
	Rpy : http://rpy.sourceforge.net/
	

Usage : ./pdtrending.py -f \'file_list\' [OPTION...]
	
	-f : list of files which you want to de-trend. You can use regular expression. Don't forget for surrounding file_list with \'. i.e. -f \'/home/astro/hht_[0-9]*.lc\'
			
	-c : Correlation matrix file if you have one. i.e. -c /home/astro/corr.dat
			
	-o : Output directory. Default is './pdtrend' under the directory where light-curves are. i.e. -o /home/astro/pdtrend/
			
	-l 0 : Column number of flux in light-curves. Default is 0 (first column).
			
	-i 1 : Not implemented in this version. Number of iteration of de-trending. If no clusters are found, the iteration will stop regardless the number of iteration. Note that png files of determined-trends trends-position will be generated only at the first iteration. Default is 1, which means no iteration but single execution of de-trending. You cannot use this option with -c together.
			
	-d : X and Y coordinates of light-curves. First(second) column should be X(Y) coordinates. The order of stars should be in same order with file_list. i.e. -c /home/astro/xy.dat
			
	-b [True|False] : True if light-curves are in flux based. If they are in magnitude based, then False. Default is True.
			
	-z 20 : Zero magnitude. Default is 20.
			
	-n 0 : Binning light-curves just when calculating correlation values, which can help to increase signal-to-noise ratio. The binning might be useful when low frequency trends are contaminated by high frequency noise source. Default is 0, which means no binning. Note that light-curves will be de-trended in original sampling rate (without binning).
			
	-t 2 : Constrain initial seeds. Default is 2. From 1 to 10 is suitable. If no clusters are found, decrease this number. It might help to find clusters. See Kim et al. 2008 for more details.
	
	-g 0.1 : level of significance, p-value. Default is 0.1. Smaller p-value generally gives more clusters.
			
	-m 5 : Mimimum number of stars in a cluster. Default is 5. According to the number of stars in dataset, you can adjust this value.
			
	-s [True|False] : If True, you can see a dendrogram of hierarchical tree of similarity matrix derived using light-curves. Default is False.
			
	-h : Show this help.
	

mailto : dakim@cfa.harvard.edu
'''

change_log = '''
@ Change log :
		v1.00 at 01/05/2009 : Just change the version number to 1.00 to publish this package.
		
		v4.00 at 11/14/2008 : We gave up multiple linear regression which occassionally destroys intrinsic signals.
		Instead of that, we implemented quadratic program which can constraint free parameters to be bigger than or equal to 0.
		This greatly reduces the signal destruction effects. Another big jump of our algorithm.
		This is our last improvement at this version of De-trending algorithm.
		Next version might start from handling the phase shift of trends.
		
		v3.51 at 09/22/2008 : Save the standard deviation values of light curves after- and before de-trending.
		
		v3.50 at 09/15/2008 : Fisher's tranformation is applied to the distances lists of subsets.
		The transformation convert skewed distribution of correlations to normal distribution;
		which strengthen our normality assumption. Nevertheless, the resulting clusters are almost
		identical because we gather only highly correlated elements and cut outliers by p-value.
		This is the thrid big jump of our algorithm.
		
		v3.03 at 08/25/2008 : Remove Shapiro-Wilk test. Use only Anderson-Darling test.
		
		v3.01 at 08/19/2008 : Save the determined trends light curves as well; as 'trend_[0-9][0-9]'
		
		v3.00 at 08/13/2008 : Clustering algorithm changed to the normality test algorithm.
		Shapiro-Wilk test (Shapiro & Wilk 1965) and Anderson-Darling test (Anderson & Darling 1952)
		is used to test the normality of cluster. Many simulation had performed again.
		New algorithm idenfity the clusters much better than before.
		The second big jump of our de-trending algorithm.
		
		v2.21 at 06/10/2008 : Software is packaged at the first time. Remove not-used function.
		Make simulation light curves for test run. Packaging the light curves separately.
		
		v2.20 at 06/01/2008 : Improvement of combining template set routine to build trend,
		we use weighting according to the standard deviation of normalized light curves.
		
		v2.10 at 05/28/2008 : Improvement of removal of trends routine. Based on the constructed trends,
		Applied multiple regression method.
		
		v2.01 at 05/22/2008 : Save constructed trends light curves as png image. If xy_coords is not null,
		then also save position of each trend on CCD plane as png image.
		
		v2.00 at 05/21/2008 : Change block-clustering algorithm to hierachichal tree clustering algorithm.
		It's one of the biggest changes in our algorithm.
		
		v1.00 was finished at 05/08/2008 by Dae-Won Kim.
'''

import getopt
#import matplotlib
#matplotlib.use('agg')
from pylab import *
from numpy import *
import os
import sys
import glob

from Pycluster import *
from hcluster import *
from clusterlib import *

sys.path.append('./')

def pdetrending(file_list, correlation_matrix='', xy_coords='', output_dir='', \
img=True, show_tree=False, min_template_stars=0, std_save=True, \
flux_based=True, zero_mag=20., column_flux=0, bin_window=0, initial_seed=2, \
n_iteration=1, l_significance=0.1):
	'''
	flie_list :
		list of files. This list should be sorted by brightness of stars.
		Each file has to contain flux list of a star at the first column.
	
	correlation_matrix = '' : 
		Correlation matrix of stars in file_list. If this is '', it will be generated by our routine.
		Newly generated matrix will be saved under the output_dir as 'corr.dat'
	
	xy_coords = '' :
		x and y coordinates of star on CCD plane. This list shoud be sorted by brightness of stars either.
		Therefore total number of lines should be the same with total number of lines of file_list.
		The first column should be x coordinate. The second column should be y coordinate.
		If this coordinates file is provided and img=True, then we save the png file which show where each
		constructed trends are placed on CCD plane. It's named 'trends_position.png' in output_dir.
		
	output_dir = '' : 
		Output directory where de-trended light curves will be saved.
		If this is '', de-trended light curves will be saved under the directory of file_list[0], 'pdtrend'
	
	img = False : 
		Save the image of de-trended light curves as png files. N number of png files will be generated
		under output_dir. N is the total number of stars.
		Top panel is raw, middle is constructed trend and bottom panel is de-trended resules.
		Also constructed trends image file will be saved as 'constructed_trends.png'
	
	show_tree = False :
		If it's true, this routine shows the dendrogram constructed based on the correlation matrix.
	
	min_template_stars = 0:
		Minimum number of template stars per cluster. Empirically, about (total number of star) / 50 is acceptable.
		However, if this value is smaller than 5, it means that constructed trends might suffer from noise
		because the number of template star is too small. If this value is set to 0, then (total number of star) / 50
		is used.
	
	std_save = True:
		Save stadnard deviation values of light curves (before and after de-trending).
	
	flux_based = True:
		If data is in magnitude, change this values to False and set zero_mag to proper values.
		
	zero_mag = 20:
		Only activated when flux_based = False.
	
	column_flux = 0:
		Column of flux in light-curve files.
	
	bin_window = 0:
		Bin light-curves when calculating correlation values. This can help to find clusters when trends are contaminated by noise a lot. 0 means no binning.
		
	initial_seed = 2:
		Constraint initial seeds. For more details, see Kim et al. 2008
		
	n_iteration = 3:
		Number of iteration of de-trending processes. Not implemented in this version.
	'''
	
	#Append current directory for python libraries
	sys.path.append('./')
	
	#Check initial values.
	if file_list == '':
		print '#ERROR : Check the file_list.'
		sys.exit()
	if output_dir == '':
		output_dir = os.path.dirname(file_list[0]) + '/pdtrend/'
	if not os.path.exists(output_dir):
		os.makedirs(output_dir)
	
	#read file into memory.
	if xy_coords != '':
		print '#Start to read xy coordinates..'
		whole_xy = []
		fp = open(xy_coords)
		whole_xy = array([map(float, line.split()) for line in fp])
		max_x = max(whole_xy[:,0])
		max_y = max(whole_xy[:,1])
		fp.close()
		if len(whole_xy) != len(file_list):
			print '#ERROR : The number of xy coords file is not matched with the number of stars.'
			sys.exit()
		
	print '#Start to read file..'
	whole_lc = []
	for i in range(len(file_list)):
		print '	#Reading ' + file_list[i] + '..'
		lc = []
		fp = open(file_list[i])
		for line in fp:
			if line[0] == '#':
				continue
			flux_column = float(line.split()[column_flux])
			if not flux_based:
				flux = math.pow(10, (flux_column - zero_mag) / -2.5)
			else:
				flux = flux_column
			lc.append(flux)
		fp.close()
		whole_lc.append(lc)
	whole_lc = array(whole_lc)
	
	#Check correlation matrix.
	if correlation_matrix == '':
		print '#Calculate correlation matrix..'
		corr_list = cal_correlation_matrix(whole_lc, bin_window)
		
		print '#Save correlation matrix to local file to %s..' % (output_dir + '/corr.dat')
		fp = open(output_dir + '/corr.dat', 'wb')
		for ele in corr_list:
			fp.write(' '.join(map(str, ele)) + '\n')
		fp.close()
	else:
		print '#Read correlation matrix..'
		fp = open(correlation_matrix)
		corr_list = array([map(float, line.split()) for line in fp])
		if corr_list.shape != (len(whole_lc), len(whole_lc)):
			print '#ERROR : Dimension of correlation matrix is not match!. \
Execute this routine without correlation matrix value and generate correlation matrix again.'
			sys.exit()
	
	#select template clusters based on the hierarchical tree structure.
	#for more details, see Kim et al. 2008.
	print '#Now finding clusters for template set..'
	dist_matrix = 1. - corr_list
	print '	#Making Hierarchical tree..'
	tree = treecluster(method='m', distancematrix=dist_matrix.copy())
	if show_tree:
		R = dendrogram(Pycluster_to_hcluster(tree), leaf_font_size=10)
		ylabel('Distance'); xlabel('Index of star'); show()
		
	print '	#Finding clusters in tree..'
	groups = find_group_DW(tree, dist_matrix, len(whole_lc[0]), initial_seed, l_significance)
	print '	#Remove subset of clusters..'
	groups = remove_subset_cluster(groups)
	groups = remove_small(groups, min_template_stars - 1)
	print '	#Total %d clusters found..' % (len(groups))
	#print groups
	for group in groups:
		print '#----------------------------#'
		for ele in group:
			print file_list[ele]
	
	if len(groups) == 0:
		print '#ERROR : No cluster is found; which means this dataset has no \
		reliable template set. You have to check below things. \
		1) Are your data contaminated by other noise sources too much? if yes, retry this algorithm after data binning. \
		2) Do you have enough number of stars? if yes, there is no solution. Our algorithm is \
		developed for wide field survey; which means you should have enough number of stars, \
		~ a few hundreds. \
		3) Do your data have trends? In other words, do your stars just show independent trends with others? \
		Check the light curves by eyes. If the trends appear just in tiny number of stars, \
		then it\'s very hard to find reliable clusters. However, you could retry this algorithm after data binning, \
		which might help to find cluters.'
		sys.exit()
	
	#make trend list of each group.
	print '#Construct trends with identified clusters..'
	trend_set = []
	for group in groups:
		trend_set.append(create_trend(group, whole_lc))
	trend_set = array(trend_set)
		
	if img:
		clf()
		color=['r', 'g', 'b', 'y', 'k']
		#color = ['0.5', '0.6', '0.7', '0.8', '0.9']
		shape=['s', '^', 'o', 'D', '+', 'x', '1', 'h', 'p']
		print '	#Saving constructed trends as png files..'
		for i in range(len(trend_set)):
			ax = subplot(len(trend_set), 1, i + 1)
			#plot(trend_set[i] + 1., color[i%len(color)]+shape[i%len(shape)])
			plot(trend_set[i] + 1., 'b,')
			ylabel('%d' % len(groups[i]))
		xlabel('Time index')
		savefig(output_dir + '/constructed_trends.png')
		
		if xy_coords != '':
			clf()
			print '	#Saving position of trends on CCD as png files..'
			for i in range(len(groups)):
				for ele in groups[i]:
					plot([whole_xy[ele][0]], [whole_xy[ele][1]], color=color[i%len(color)], marker=shape[i%len(shape)])
			axis([min(whole_xy[:,0]), max(whole_xy[:,0]), min(whole_xy[:,1]), max(whole_xy[:,1])])
			xlabel('x-coordinate on CCD plane')
			ylabel('y-coordinate on CCD plane')
			savefig(output_dir + '/trends_position.png')
	
	#Start de-trending.
	print '#Start to de-trend every light curves..'
	#standard deviation list of before and after de-trending
	std_list = []
	for i in range(len(whole_lc)):
		print '	#Now de-trending %dth star.. %s' %((i + 1), file_list[i])
		detrended = get_quadprog(whole_lc[i] / median(whole_lc[i]) - 1. , trend_set)
		detrended += 1.
		detrended *= median(whole_lc[i])
		#detrended = get_quadprog(whole_lc[i] , trend_set)
		print '		#Save the de-trended light curve..'
		fp = open(output_dir + '/' + os.path.basename(file_list[i]), 'w')
		if len(trend_set) == 1:
			detrended = detrended[0]
		for flux in detrended:
			fp.write('%.2f\n' % (round(flux, 2)))
		fp.close()
		if std_save:
			std_list.append([std(whole_lc[i]), std(detrended)])
		
		if img:
			print '		#Save as image file..'
			clf()
			ax = subplot(211)
			title(os.path.basename(file_list[i]))
			plot(whole_lc[i] / median(whole_lc[i]), 'b.')
			text(0.98, 0.9, r'Raw, $\sigma$ : %.1f' % std(whole_lc[i]), horizontalalignment='right', \
			verticalalignment='center', transform=ax.transAxes, fontsize=10, \
			bbox=dict(facecolor='blue', alpha=0.5))
			ylabel('Flux')
			ax = subplot(212)
			plot(detrended / median(whole_lc[i]), 'g.')
			text(0.98, 0.9, r'De-trended, $\sigma$ : %.1f' % std(detrended), horizontalalignment='right', \
			verticalalignment='center', transform=ax.transAxes, fontsize=10, \
			bbox=dict(facecolor='green', alpha=0.5))
			if flux_based:
				ylabel('Normalized Flux')
			else:
				ylabel('Normalized Magnitude')
			xlabel('Time indices')
			savefig(output_dir + '/' + os.path.basename(file_list[i]) + '.png')
	
	if std_save:
		print '#Save standard deviation values..'
		fp = open(output_dir + '/std_list', 'w')
		for i in range(len(std_list)):
			fp.write('%.2f %.2f %s\n' % (round(std_list[i][0], 2), round(std_list[i][1], 2), file_list[i]))
		fp.close()
		
if __name__=='__main__':
	
	file_l = '' # file list of light-curves.
	correlation_matrix = '' # correlation matrix file.
	xy_coords = '' # x and y coordinates of all stars.
	output_d = '' # output directory.
	column_f = 0 # colun number of flux in light-curve files.
	flux_b = True # flux based or magnitude based.
	zero_m = 20 # zero magnitude when light-curves are in magnitude based.
	show_t = False # show dendrogram of hierarchical tree.
	ini_s = 2 # initial seeds constraint.
	bin_w = 0 # binning window size.
	min_t = 5 # minimum number of stars in clusters. 5~10 is good choise.
	n_iter = 1
	l_sig = 0.1
	
	if len(sys.argv) == 1:
		print help
		sys.exit()
	
	#read command line option.
	try:
		optlist, args = getopt.getopt(sys.argv[1:], 'h:f:c:d:b:z:s:o:l:b:t:i:m:g:')
	except getopt.GetoptError, err:
		print help
		sys.exit()
	
	for o, a in optlist:
		if o in ('-h'):
			print help
			sys.exit()
		elif o in ('-f'):
			file_l = a
		elif o in ('-c'):
			correlation_matrix = a
		elif o in ('-d'):
			xy_coords = a
		elif o in ('-b'):
			if a == 'False':
				flux_b = False
			else:
				flux_b = True
		elif o in ('-z'):
			zero_m = float(a)
		elif o in ('-s'):
			show_t = a
		elif o in ('-o'):
			output_d = a
		elif o in ('-l'):
			column_f = int(a)
		elif o in ('-n'):
			bin_w = int(a)
		elif o in ('-t'):
			ini_s = float(a)
		elif o in ('-i'):
			n_iter = int(a)
		elif o in ('-m'):
			min_t = int(a)
		elif o in ('-g'):
			l_sig = float(a)
		else:
			continue
		
	file_list = glob.glob(file_l)
	if len(file_list) == 0:
		print 'There is no file in : ' + file_l
		sys.exit()
	if len(file_list) < 50:
		print 'Too few light curves : %d.\nIf you still want to run the program, modify line#406 in pdtrending.py.' % (len(file_list))
		sys.exit()
	file_list.sort()
	
	pdetrending(file_list, correlation_matrix, xy_coords, show_tree=show_t, \
		min_template_stars=min_t, flux_based=flux_b, zero_mag=zero_m, output_dir=output_d, \
		column_flux=column_f, bin_window=bin_w, initial_seed=ini_s, n_iteration=n_iter, l_significance=l_sig)
	


import numpy as np
import warnings

from scipy.interpolate import UnivariateSpline


class FMdata:
    """
    Filling missing values using linear interpolation.

    Extrapolation should be avoided, and thus we sync
    each light curve based on the latest start epoch and the earliest end
    epoch among the all light curves.

    Parameters
    ----------
    lcs : array_like
        An array containing a set of light curves. The number of data points
        in each light curve does not need to be identical.
    times : array_like
        An array of observation epochs of the corresponding light curve.
    n_min_data : int, optional
        The minimum number of data points in each light curve.
        If fewer than this, the light curve will be discarded.
    """
    def __init__(self, lcs, times, n_min_data=100):
        # Type check.
        if type(lcs) != np.ndarray:
            lcs = np.array(lcs)
        if type(times) != np.ndarray:
            times = np.array(times)

        # Dimension check.
        if lcs.shape[0] != times.shape[0]:
            raise RuntimeError('The number of light curves and ' +
                               'the number of times do not match.')

        # Discard light curves having fewer data points than "n_min_data".
        keep_index = []
        for i in range(len(lcs)):
            if len(lcs[i]) >= n_min_data:
                keep_index.append(i)

        # Initialize.
        self.lcs = lcs[keep_index]
        self.times = times[keep_index]

    def run(self):
        """
        Fill missing values and returns results.

        Returns
        -------
        results : dict
            A Python dictionary containing three items such as
            missing-value filled light curves,
            synced epoch, and the indices of corresponding
            raw light curves.
        """
        # Sync times.
        self._sync_time()

        # Fill missing values.
        self._fill_missing_values()

        # Returned output is a Python dictionary containing three items.
        results = {'lcs': self.filled_lcs, 'epoch': self.synced_epoch,
                   'indices': self.indices}
        return results

    def _sync_time(self):
        """
        Walk through times of all light curves and create one-dimensional
        list of times that will be used to fill missing values for every light
        curves. In order to prevent extrapolation, we chose the latest start
        epoch and the earliest end epoch as the new epoch range.
        """
        # Get all unique epochs and find
        # the latest start epoch and the earliest end epoch.
        all_epoch = []
        latest_start_epoch = -np.inf
        earliest_end_epoch = np.inf
        for t in self.times:
            all_epoch = np.hstack([all_epoch, t])
            if t[0] > latest_start_epoch:
                latest_start_epoch = t[0]
            if t[-1] < earliest_end_epoch:
                earliest_end_epoch = t[-1]

        all_epoch = np.unique(np.ravel(all_epoch))
        all_epoch.sort()

        # Cut epoch.
        start_index = np.searchsorted(all_epoch, latest_start_epoch)
        end_index = np.searchsorted(all_epoch, earliest_end_epoch) + 1
        epoch = all_epoch[start_index:end_index]

        self.synced_epoch = epoch

    def _fill_missing_values(self):
        """Fill missing values for each light curve."""
        filled_lcs = []
        filled_indices = []
        for i in range(len(self.lcs)):
            # Check if the length is same. Print warning but not break the loop.
            # Thus, need to discard the corresponding weight as well.
            if len(self.times[i]) != len(self.lcs[i]):
                warnings.warn(('The number of data points of the %dth ' +
                               'light curve is not matched with the one ' +
                               'of the list of times, and thus is discarded.')
                              % (i + 1))
                continue

            # Linear spline fitting without smoothing. Thus the fitted line
            # follows the exact values of the given input data.
            spl = UnivariateSpline(self.times[i], self.lcs[i], k=1., s=0.)
            filled_lc = spl(self.synced_epoch)

            # Add the filled lc to a list.
            filled_lcs.append(filled_lc)
            # Indices for the new list.
            filled_indices.append(i)

        self.filled_lcs = np.array(filled_lcs)
        self.indices = np.array(filled_indices)



"""
Base IO code for all datasets
"""

import sys

from os.path import dirname
from os.path import join


def load_lightcurve_set():
    """
    Return the set of light curves for testing pdtrend.

    Returns
    -------
    lcs : numpy.ndarray
        An array of light curves.
    """

    import bz2

    try:
        import cPickle as pickle
    except:
        import pickle

    module_path = dirname(__file__)

    # The light curves are bzipped and pickled.
    file_path = join(module_path, 'lightcurves/lc.pbz2')
    # For Python 3.
    if sys.version_info.major >= 3:
        lcs = pickle.load(bz2.BZ2File(file_path, 'r'), encoding='bytes')
    # For Python 2.
    else:
        lcs = pickle.load(bz2.BZ2File(file_path, 'r'))

    return lcs



import numpy as np

from sklearn.cluster import Birch
from scipy.optimize import least_squares


class PDTrend:
    """
    Detrending systematic trends in a set of light curves.

    PDTrend determines master trends using Pearson correlation coefficients
    between each light curve and using machine learning, and then provide a
    function to detrend each light curve using the determined master trends.
    For details, see Kim et al. 2009 or visit the GitHub repository
    (https://goo.gl/uRAXfr).

    Parameters
    ----------
    lcs : array_like
        A list of light curves. It must be the shape of N x M,
        where N is the number of light curves and M is the number of
        data points. Note that M must be same for all N light curves.
        pdtrend assumes that these M data points are synced in time.
        Thus pdtrend does not use any time information.
    weights : array_like, optional
        A list of weights for the corresponding light curves.
        It is used only when constructing master trends. It must contain N
        elements. N is the number of the light curves. See the "lcs" parameter.
        Default is None, so the identical weights for all light curves.
    xy_coords : array_like, optional
        X and Y coordinates of stars of the light curves.
        It must contain Nx2 elements, where N is the number of the light curves.
        If the coordinates are given, the function "plot_spatial" can be called
        after constructing master trends, which will plot the spatial
        distribution of the master trends.
    n_min_member : int, optional
        The minimum number of members in each cluster. Default is 10.
    dist_cut : float, optional
        Distance cut to filter clusters found using the Birch clustering
        algorithm. If the median distance
        between members in a cluster is larger than the cut,
        the cluster is discarded. Must be between [0, 1]. Default is 0.45.
    branching_factor : int, optional
        Branching factor for the Birch clustering. Default is 50.
    threshold : float, optional
        Threshold for the Birch clustering. Default is 0.5.
    """
    def __init__(self, lcs, weights=None, xy_coords=None,
                 n_min_member=10, dist_cut=0.45,
                 branching_factor=50, threshold=0.5):
        # Convert the light curve set to numpy array.
        if type(lcs) != np.ndarray:
            lcs = np.array(lcs)

        # Sanity check.
        if len(lcs.shape) != 2:
            raise RuntimeError('lcs must be a 2-dimensional array.')

        if lcs.shape[0] < n_min_member:
            raise RuntimeError('The number of light curves in lcs ' +
                               'is fewer than the n_min_member.')
        if weights is not None:
            if type(weights) != np.ndarray:
                weights = np.array(weights)

            if lcs.shape[0] != weights.shape[0]:
                raise RuntimeError('Shapes of lcs and weights do not match.')
        else:
            # Same weights for the all light curves.
            weights = np.ones(lcs.shape[0])

        if xy_coords is not None:
            if type(xy_coords) != np.ndarray:
                xy_coords = np.array(xy_coords)

            if lcs.shape[0] != xy_coords.shape[0]:
                raise RuntimeError('Shapes of lcs and xy_coords do not match.')

        # Set parameters.
        self.lcs = lcs
        self.weights = weights
        self.xy_coords = xy_coords
        self.n_min_member = n_min_member
        self.dist_cut = dist_cut

        # Initialize.
        self.corr_matrix = None
        self.dist_matrix = None
        self.birch = None
        self.branching_factor = branching_factor
        self.threshold = threshold

    def _calculate_distance_matrix(self):
        """
        Calculate a distance matrix, which is defined as:
        (1. - correlation_matrix) / 2.
        """
        corr_matrix = np.corrcoef(self.lcs)
        dist_matrix = (1. - corr_matrix) / 2.

        self.corr_matrix = corr_matrix
        self.dist_matrix = dist_matrix

    def _find_clusters(self):
        """Find clusters using Birch and the distance matrix."""
        # TODO: Need to test with different threshold.
        # Need to test with multiple dataset having trends.
        # Branching factor is fine.
        birch = Birch(branching_factor=self.branching_factor,
                      threshold=self.threshold,
                      n_clusters=None).fit(self.dist_matrix)

        self.birch = birch

    def _filter_clusters(self):
        """
        Discard a cluster if 1) it has less than "n_min_member" members, or
        2) median distance between each member is larger than "dist_cut".
        """
        unique_labels = set(self.birch.labels_)
        _filtered_labels = []

        for label in unique_labels:
            index = [i for i in range(len(self.birch.labels_)) if
                     self.birch.labels_[i] == label]
            # The number of members in the given cluster.
            if len(index) < self.n_min_member:
                continue

            dist_list = []
            for i in range(len(index) - 1):
                for j in range(i + 1, len(index)):
                    dist_list.append(self.dist_matrix[index[i], index[j]])

            # Median distance check.
            if np.median(dist_list) <= self.dist_cut:
                _filtered_labels.append(label)

        self._filtered_labels = _filtered_labels

        # Check how many clusters are left.
        if len(self._filtered_labels) == 0:
            raise RuntimeWarning(
                'No clusters were found. ' +
                'Adjust input parameters and try again. ' +
                'For instance, decrease "n_min_member" or ' +
                'increase "dist_cut". For details, ' +
                'visit https://github.com/dwkim78/pdtrend'
            )

    def _build_master_trends(self):
        """Build master trends using the filtered clusters."""
        master_trends_indices = []
        master_trends = []
        for label in self._filtered_labels:
            index = [i for i in range(len(self.birch.labels_)) if
                     self.birch.labels_[i] == label]
            master_trends_indices.append(index)

            trends = []
            weights_sum = 0
            for i in range(len(index)):
                # Normalization.
                med_lc = np.median(self.lcs[index[i]])
                normed = (self.lcs[index[i]] - med_lc) / med_lc

                # Weights.
                weights = self.weights[index[i]]
                normed *= weights
                weights_sum += weights

                trends.append(normed)

            # Construct a master trends using the normalized and weighted
            # light curves from the above for loop.
            master_trends.append(np.sum(trends, axis=0) / weights_sum)

        self.master_trends_indices = master_trends_indices
        self.master_trends = master_trends

    def run(self):
        """Run pdtrend pipeline."""
        if self.corr_matrix is None or self.dist_matrix is None:
            self._calculate_distance_matrix()
            self._find_clusters()
        else:
            # For the safety, just in case.
            # If corr_matrix and dist_matrix is not None,
            # birch must be not None either.
            if self.birch is None:
                self._find_clusters()

        self._filter_clusters()
        self._build_master_trends()

    def _func_trends(self, p, x):
        """Return sum of the trends."""
        return np.sum(x * p.reshape(len(p), 1), axis=0)

    def _residuals(self, p, x, y):
        """Return residual between sum of the trends and a light curve."""
        return y - self._func_trends(p, x)

    def detrend(self, lc):
        """Detrend a light curves using the constructed master trends."""

        # Convert the light curve set to numpy array.
        if type(lc) != np.ndarray:
            lc = np.array(lc)

        # Normalize.
        med_lc = np.median(lc)
        raw = (lc - med_lc) / med_lc

        # Initial guess.
        p0 = np.ones(len(self.master_trends))
        # Bounds in [0, infinite]
        p1 = least_squares(self._residuals, p0, args=(self.master_trends, raw),
                           bounds=[0, np.inf])
        p1 = p1['x']

        detrended = raw - self._func_trends(p1, self.master_trends)

        # Scale back to the original flux.
        detrended = detrended * med_lc + med_lc

        return detrended

    def plot_spatial(self, filename='spatial.png'):
        """
        Plot a spatial distribution of the constructed master trends.

        Parameters
        ----------
        filename : str, optional
            A png filename including the path. For example,
            "./outputs/spatial.png". Default is "spatial.png"
        """
        if self.xy_coords is None:
            raise RuntimeError('No x and y coordinates are given.')

        import pylab as pl

        pl.figure(figsize=(12, 12))
        pl.title('Spatial distribution of the constructed master trends')

        colors = 'bgrkmc'
        marks = 's^.+x*'
        for i in range(len(self.master_trends_indices)):
            indices = self.master_trends_indices[i]
            pl.plot(self.xy_coords[indices][:, 0],
                    self.xy_coords[indices][:, 1],
                    marker=marks[int(i / len(colors)) % len(marks)],
                    color=colors[i % len(colors)], ls='None',
                    label='Master trend %d: %d light curves' %
                          (i + 1, len(self.master_trends_indices[i])))
        pl.xlabel('X coordinate')
        pl.ylabel('Y coordinate')
        pl.legend(numpoints=1)
        pl.savefig(filename)



from setuptools import find_packages, setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(
    name='pdtrend',
    version='0.2.3',
    description='Photometric Detrending Algorithm',
    long_description=readme(),
    platforms=['any'],
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/dwkim78/pdtrend',
    license='MIT',
    author='Dae-Won Kim',
    author_email='dwkim78@gmail.com',
    install_requires=['numpy>=1.10', 'scikit-learn>=0.17', 'scipy>=0.17',
                      'matplotlib>=1.5'],
    keywords=['astronomy', 'light curves', 'time series',
              'machine learning', 'trend', 'detrend'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Scientific/Engineering :: Astronomy'
    ]
)



import os

import numpy as np
import pylab as pl

from pdtrend.filling_missing_data import FMdata
from pdtrend.detrend import PDTrend
from pdtrend.datasets.base import load_lightcurve_set
from pdtrend.utils.logger import Logger


def test_with_missing_data():
    logger = Logger().getLogger()

    logger.info('Loading the light curve set.')
    lcs = load_lightcurve_set()
    logger.info('\tThe number of light curves is %d.' % len(lcs))

    # Create time information.
    times = np.ones(lcs.shape) * np.arange(lcs.shape[1])

    logger.info('Make missing data points.')
    # The maximum ratio of missing data points.
    np.random.seed(2)
    missing_ratio = 1.0
    lcs_new = []
    times_new = []
    for i in range(len(lcs)):
        n_missing = int(np.random.rand() * missing_ratio * len(lcs[i]))
        index = np.arange(0, len(lcs[i]))
        np.random.shuffle(index)
        keep_index = index[n_missing:]
        keep_index.sort()

        lcs_new.append(lcs[i][keep_index])
        # Make wrong time information for test.
        if i == 10:
            times_new.append(times[i][keep_index[:-2]])
        else:
            times_new.append(times[i][keep_index])

    logger.info('Fill missing data points.')
    # Create random weights.
    # weights = np.random.rand(lcs.shape[0])
    # Same weights.
    weights = np.ones(lcs.shape[0])

    # Fill missing values.
    fmd = FMdata(lcs_new, times_new)
    results = fmd.run()

    # The output, "results", contains the following three items.
    lcs = results['lcs']
    epoch = results['epoch']
    indices = results['indices']

    # X and Y coordinates, randomly generated.
    xy_coords = np.random.rand(lcs.shape[0], 2) * 1000.

    logger.info('Initializing pdtrend.')
    pdt = PDTrend(lcs, weights=weights[indices], xy_coords=xy_coords,
                  n_min_member=10, dist_cut=0.45)

    # We can run all the following protected routines using "pdtrend.run()",
    # but, here we do it individually to print log messages.
    logger.info('Calculating the distance matrix.')
    pdt._calculate_distance_matrix()
    logger.info('Searching for clusters using Birch.')
    pdt._find_clusters()
    logger.info('Filtering the clusters.')
    pdt._filter_clusters()
    logger.info('Building master trends.')
    pdt._build_master_trends()
    logger.info('Detrending a light curve using the master trends.')
    detrended = pdt.detrend(lcs[1])

    logger.info('Plotting results.')
    # Creating an output folder.
    output_path = './outputs/'
    if not os.path.exists(output_path):
        os.makedirs(output_path)

    # Plotting spatial distribution.
    pdt.plot_spatial('%s/spatial.png' % output_path)

    # Plotting the master trends.
    pl.figure(figsize=(10, 4))
    for i in range(len(pdt.master_trends)):
        pl.subplot(len(pdt.master_trends), 1, i + 1)
        pl.plot(epoch, pdt.master_trends[i], 'b.', label='Master trend %d' % (i + 1))
        pl.ylabel('Normalized flux')
        pl.xlabel('Time index')
        pl.legend(numpoints=1, loc='lower right')
        pl.grid()
    pl.tight_layout()
    pl.savefig('%s/master_trends.png' % output_path)

    # Plotting a detrended result of one light curve.
    pl.figure(figsize=(14, 8))
    pl.subplot(211)
    pl.plot(epoch, lcs[1], 'b.', label='Raw')
    pl.text(8955, 62200, r'$\sigma$: %.1f' % np.std(lcs[1]), fontsize=15,
            va='center', bbox=dict(boxstyle='round', ec='w', fc='g', alpha=0.3))
    pl.ylabel('Flux')
    pl.grid()
    pl.legend(numpoints=1)

    pl.subplot(212)
    pl.plot(epoch, detrended, 'b.', label='Detrended')
    pl.text(8955, 58300, r'$\sigma$: %.1f' % np.std(detrended), fontsize=15,
            va='center', bbox=dict(boxstyle='round', ec='w', fc='g', alpha=0.3))
    pl.ylabel('Flux')
    pl.xlabel('Time index')
    pl.grid()
    pl.legend(numpoints=1)
    pl.savefig('%s/detrended.png' % output_path)

    logger.info('Done.')
    logger.handlers = []


if __name__ == '__main__':
    test_with_missing_data()


#!/usr/bin/env python

'''
	Small function set for various usages.
'''

from numpy import *

def sigma_clipping(x, sigma=3., iteration=1):
	'''
	Sigma clipping of x array. Replaced the values over the
	sigma limit with the median values of x array.
	
	x :
		Array of elements.
	
	sigma :
		Threshold.
		
	iteration :
		Number of iteration of sigma clipping. 1~3 is desirable.
	
	return :
		Sigma clipped array of x
	'''
	
	x = array(x)
	xx = x.copy()
	for i in range(iteration):
		median_val = median(x)
		std_val = std(x)
		xx[where(fabs(x - median_val) > std_val * sigma)] = median_val
	return xx

def cmp_length(l, m):
	'''
		Compare the length of two input elements.
		For sort function in length.
	'''
	return len(l) - len(m)

def bin_lc(x, window_size=10, axis=None):
	'''Binning array and return new shortened array
	
	x : 
		original series.
	
	window_size = 10 : 
		default window size.
	
	axis = None : 
		index of x
		
	return :
		list of [bin_lc, std_lc, new_axis if axis != None]
	'''
	x = array(x)
	new_lc = []
	std_lc = []
	for i in range(int(len(x) / window_size) + 1):
		start_index = i * window_size
		end_index = (i + 1) * window_size
		if end_index >= len(x):
			break
		new_lc.append(mean(x[start_index:end_index]))
		std_lc.append(std(x[start_index:end_index]))
	
	if axis != None:
		new_axis = []
		for i in range(int(len(axis) / window_size) + 1):
			start_index = i * window_size
			end_index = (i + 1) * window_size
			if end_index >= len(axis):
				break
			new_axis.append(mean(axis[start_index:end_index]))
	
	if axis != None:
		return [array(new_lc), array(std_lc), array(new_axis)]
	else:
		return [array(new_lc), array(std_lc)]
		
if __name__=='__main__':
	print 'Function set for various usages'
